@@ -23,12 +23,12 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 #include "IOFireWireAVCProtocolUserClient.h"
-#include "IOFireWireAVCConsts.h"
+#include <IOKit/avc/IOFireWireAVCConsts.h>
 #include <IOKit/firewire/IOFireWireNub.h>
 #include <IOKit/firewire/IOFireWireBus.h>
 #include <IOKit/firewire/IOFWAddressSpace.h>
-#include "IOFireWirePCRSpace.h"
-#include "IOFireWireAVCRequestSpace.h"
+#include <IOKit/avc/IOFireWirePCRSpace.h>
+#include <IOKit/avc/IOFireWireAVCTargetSpace.h>
 
 // Structure for keeping track of allocated plugs
 struct PlugInfo {
@@ -118,6 +118,55 @@ IOExternalMethod IOFireWireAVCProtocolUserClient::sMethods[kIOFWAVCProtocolUserC
         kIOUCScalarIScalarO,
         2,
         0
+    },
+    { //    kIOFWAVCProtocolUserClientPublishAVCUnitDirectory
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::publishAVCUnitDirectory,
+        kIOUCScalarIScalarO,
+        0,
+        0
+    },
+    { //    kIOFWAVCProtocolUserClientSetSubunitPlugSignalFormat
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::setSubunitPlugSignalFormat,
+        kIOUCScalarIScalarO,
+        4,
+        0
+    },
+    { //    kIOFWAVCProtocolUserClientGetSubunitPlugSignalFormat, // kIOUCScalarIScalarO 3, 1
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::getSubunitPlugSignalFormat,
+        kIOUCScalarIScalarO,
+        3,
+        1
+    },
+    { //    kIOFWAVCProtocolUserClientConnectTargetPlugs,		// kIOUCStructIStructO
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::connectTargetPlugs,
+        kIOUCStructIStructO,
+        sizeof(AVCConnectTargetPlugsInParams),
+        sizeof(AVCConnectTargetPlugsOutParams)
+    },
+    { //    kIOFWAVCProtocolUserClientDisconnectTargetPlugs,	// kIOUCScalarIScalarO 6, 0
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::disconnectTargetPlugs,
+        kIOUCScalarIScalarO,
+        6,
+        0
+    },
+	{ //    kIOFWAVCProtocolUserClientGetTargetPlugConnection,	// kIOUCStructIStructO
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::getTargetPlugConnection,
+        kIOUCStructIStructO,
+        sizeof(AVCGetTargetPlugConnectionInParams),
+        sizeof(AVCGetTargetPlugConnectionOutParams)
+    },
+	{ //    kIOFWAVCProtocolUserClientAVCRequestNotHandled,  // kIOUCScalarIStructI 4, -1
+        0,
+        (IOMethod) &IOFireWireAVCProtocolUserClient::AVCRequestNotHandled,
+        kIOUCScalarIStructI,
+        4,
+        0xFFFFFFFF	// variable
     }
 };
 
@@ -143,6 +192,22 @@ IOExternalAsyncMethod IOFireWireAVCProtocolUserClient::sAsyncMethods[kIOFWAVCPro
         kIOUCScalarIScalarO,
         1,
         1
+    },
+	{
+		//    kIOFWAVCProtocolUserClientInstallAVCCommandHandler
+        0,
+        (IOAsyncMethod) &IOFireWireAVCProtocolUserClient::installAVCCommandHandler,
+        kIOUCScalarIScalarO,
+        4,
+        0
+    },
+	{
+		//    kIOFWAVCProtocolUserClientAddSubunit
+        0,
+        (IOAsyncMethod) &IOFireWireAVCProtocolUserClient::addSubunit,
+        kIOUCScalarIScalarO,
+        5,
+        1
     }
 };
 
@@ -153,75 +218,107 @@ void IOFireWireAVCProtocolUserClient::forwardPlugWrite(void *refcon, UInt16 node
     void * args[kMaxAsyncArgs];
     UInt32 generation;
     UInt16 localID;
-    
+
+    //IOLog( "IOFireWireAVCProtocolUserClient::forwardPlugWrite\n");
+	
     // fill out return parameters
     info->fClient->fDevice->getNodeIDGeneration(generation, localID);
     args[0] = info->fUserRefcon;
     args[1] = (void*)generation;
-    args[2] = (void*)nodeID;
+    args[2] = (void*)((UInt32)nodeID);
     args[3] = (void*)plug;
     args[4] = (void*)oldVal;
     args[5] = (void*)newVal;
     info->fClient->sendAsyncResult( ((PlugInfo *)info)->fCallbackInfo, kIOReturnSuccess, args, 6 );
 }
 
-UInt32 IOFireWireAVCProtocolUserClient::avcRequestHandler(void *refcon, UInt16 nodeID, IOFWSpeed &speed,
-                                  FWAddress addr, UInt32 len, const void *buf, IOFWRequestRefCon requestRefcon)
+void IOFireWireAVCProtocolUserClient::avcTargetCommandHandler(const AVCCommandHandlerInfo *pCmdInfo,
+															  UInt32 generation,
+															  UInt16 nodeID,
+															  const void *command,
+															  UInt32 cmdLen,
+															  IOFWSpeed &speed,
+															  UInt32 handlerSearchIndex)
 {
-    IOFireWireAVCProtocolUserClient * me = (IOFireWireAVCProtocolUserClient *)refcon;
-    
-    // Send stuff off to user land
-    if( me->fAVCRequestCallbackAsyncRef[0] != 0 ) {
-        void * args[kMaxAsyncArgs];
-        UInt32 generation;
-        UInt16 localID;
-        const UInt8 *src;
-        UInt8 *dst;
-        UInt32 copyLen;
-        
-        // fill out return parameters
-        me->fDevice->getNodeIDGeneration(generation, localID);
-        args[0] = 0;		// Initial call
-        args[2] = (void*)generation;
-        args[3] = (void*)nodeID;
-        args[4] = (void*)len;
+    void * args[kMaxAsyncArgs];
+	OSAsyncReference asyncRef;
+	const UInt8 *src;
+	UInt8 *dst;
+	UInt32 copyLen;
+	
+    //IOLog( "IOFireWireAVCProtocolUserClient::avcTargetCommandHandler\n");
+
+	bcopy(pCmdInfo->asyncRef, asyncRef, sizeof(OSAsyncReference));
+
+	args[0] = 0;		// Initial call
+	args[2] = (void*)generation;
+	args[3] = (void*)((UInt32)nodeID);
+	args[4] = (void*)cmdLen;
+	args[5] = (void*)pCmdInfo->userCallBack;
+	args[6] = (void*)pCmdInfo->userRefCon;
+	args[7] = (void*)speed;
+	args[8] = (void*)handlerSearchIndex;
+	
+	src = (const UInt8*)command;
+	dst = (UInt8 *)(args+9);
+	copyLen = cmdLen;
+	if(copyLen > (kMaxAsyncArgs - 9)*sizeof(void *))
+		copyLen =  (kMaxAsyncArgs - 9)*sizeof(void *);
+	bcopy(src, dst, copyLen);
+	args[1] = (void*)copyLen;
+	pCmdInfo->userClient->sendAsyncResult( asyncRef, kIOReturnSuccess,
+					  args, 9+(copyLen+sizeof(void *)-1)/sizeof(void *) );
+	cmdLen -= copyLen;
+	src += copyLen;
+	// Send rest of packet if necessary
+	while(cmdLen) {
+		copyLen = cmdLen;
+		if(copyLen > (kMaxAsyncArgs - 2)*sizeof(void *))
+			copyLen =  (kMaxAsyncArgs - 2)*sizeof(void *);
+		bcopy(src, dst, copyLen);
+		args[0] = (void *)(src - (const UInt8*)command);	// Bit being sent
+		args[1] = (void*)copyLen;
+		pCmdInfo->userClient->sendAsyncResult( asyncRef, kIOReturnSuccess,
+					   args, 2+(copyLen+sizeof(void *)-1)/sizeof(void *) );
+		cmdLen -= copyLen;
+		src += copyLen;
+	}
+}
+
+void IOFireWireAVCProtocolUserClient::avcSubunitPlugHandler(const AVCSubunitInfo *pSubunitInfo,
+															IOFWAVCSubunitPlugMessages plugMessage,
+															IOFWAVCPlugTypes plugType,
+															UInt32 plugNum,
+															UInt32 messageParams,
+															UInt32 generation,
+															UInt16 nodeID)
+{
+    void * args[kMaxAsyncArgs];
+	OSAsyncReference asyncRef;
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::avcSubunitPlugHandler\n");
+
+	bcopy(pSubunitInfo->asyncRef, asyncRef, sizeof(OSAsyncReference));
+
+	args[0] = (void*) pSubunitInfo->subunitTypeAndID;
+	args[1] = (void*) plugType;
+	args[2] = (void*) plugNum;
+	args[3] = (void*) plugMessage;
+	args[4] = (void*) messageParams;
+	args[5] = (void*)pSubunitInfo->userCallBack;
+	args[6] = (void*)pSubunitInfo->userRefCon;
+	args[7] = (void*)generation;
+	args[8] = (void*)((UInt32)nodeID);
+	
+	pSubunitInfo->userClient->sendAsyncResult( asyncRef, kIOReturnSuccess,
+										args, 9);
 
-        src = (const UInt8*)buf;
-        dst = (UInt8 *)(args+5);
-        copyLen = len;
-        if(copyLen > (kMaxAsyncArgs - 5)*sizeof(void *))
-            copyLen =  (kMaxAsyncArgs - 5)*sizeof(void *);
-        bcopy(src, dst, copyLen);
-        args[1] = (void*)copyLen;
-        me->sendAsyncResult( me->fAVCRequestCallbackAsyncRef, kIOReturnSuccess,
-                                args, 5+(copyLen+sizeof(void *)-1)/sizeof(void *) );
-        len -= copyLen;
-        src += copyLen;
-        // Send rest of packet if necessary
-        while(len) {
-            copyLen = len;
-            if(copyLen > (kMaxAsyncArgs - 2)*sizeof(void *))
-                copyLen =  (kMaxAsyncArgs - 2)*sizeof(void *);
-            bcopy(src, dst, copyLen);
-            args[0] = (void *)(src - (const UInt8*)buf);	// Bit being sent
-            args[1] = (void*)copyLen;
-            me->sendAsyncResult( me->fAVCRequestCallbackAsyncRef, kIOReturnSuccess,
-                                args, 2+(copyLen+sizeof(void *)-1)/sizeof(void *) );
-            len -= copyLen;
-            src += copyLen;
-        }
-    }
-    
-    return kFWResponseComplete;
 }
 
 void IOFireWireAVCProtocolUserClient::free()
 {
-    if(fAVCSpace) {
-        fAVCSpace->deactivate();
-        fAVCSpace->release();
-    }
-    
+    //IOLog( "IOFireWireAVCProtocolUserClient::free (0x%08X)\n",(int) this);
+
     if(fInputPlugs) {
         OSData *data;
         while(data = OSDynamicCast(OSData, fInputPlugs->getAnyObject())) {
@@ -244,15 +341,22 @@ void IOFireWireAVCProtocolUserClient::free()
         fPCRSpace->deactivate();
         fPCRSpace->release();
     }
+    if(fAVCTargetSpace) {
+        fAVCTargetSpace->deactivateWithUserClient(this);
+        fAVCTargetSpace->release();
+    }
     
     IOUserClient::free();
 }
 
 bool IOFireWireAVCProtocolUserClient::start(IOService *provider)
 {
-    OSObject *prop;
+	OSObject *prop;
     IOFireWireNub *device;
-    device = OSDynamicCast(IOFireWireNub, provider->getProvider());
+
+    //IOLog( "IOFireWireAVCProtocolUserClient::start (0x%08X)\n",(int) this);
+
+	device = OSDynamicCast(IOFireWireNub, provider->getProvider());
     if(!device)
         return false;
 
@@ -269,11 +373,20 @@ bool IOFireWireAVCProtocolUserClient::start(IOService *provider)
 
     if(!fInputPlugs || !fOutputPlugs)
         return false;
+
     fPCRSpace = IOFireWirePCRSpace::getPCRAddressSpace(fDevice->getBus());
     if(!fPCRSpace)
         return false;
     fPCRSpace->activate();
-        
+
+    fAVCTargetSpace = IOFireWireAVCTargetSpace::getAVCTargetSpace(fDevice->getController());
+    if(!fAVCTargetSpace)
+        return false;
+    fAVCTargetSpace->activateWithUserClient(this);
+
+	// Enable the communication between the PCR space and the Target space objects
+	fPCRSpace->setAVCTargetSpacePointer(fAVCTargetSpace);
+	
     registerService();
     
     return true;
@@ -283,7 +396,9 @@ IOReturn IOFireWireAVCProtocolUserClient::newUserClient( task_t owningTask, void
                                     UInt32 type,  OSDictionary * properties,
                                     IOUserClient ** handler )
 {
-    if(fStarted)
+    //IOLog( "IOFireWireAVCProtocolUserClient::newUserClient (0x%08X)\n",(int) this);
+
+	if(fStarted)
         return kIOReturnStillOpen;
     retain();
     *handler = this;
@@ -298,7 +413,7 @@ IOReturn IOFireWireAVCProtocolUserClient::newUserClient( task_t owningTask, void
 
 IOReturn IOFireWireAVCProtocolUserClient::clientClose( void )
 {
-    IOLog( "IOFireWireAVCProtocolUserClient : clientClose\n" );
+    //IOLog( "IOFireWireAVCProtocolUserClient::clientClose (0x%08X)\n",(int) this);
 
 	fStarted = false;
 	
@@ -309,7 +424,7 @@ IOReturn IOFireWireAVCProtocolUserClient::clientClose( void )
 
 IOReturn IOFireWireAVCProtocolUserClient::clientDied( void )
 {
-	IOLog( "IOFireWireAVCProtocolUserClient : clientDied\n" );
+    //IOLog( "IOFireWireAVCProtocolUserClient::clientDied (0x%08X)\n",(int) this);
 
     return clientClose();
 }
@@ -319,7 +434,9 @@ IOReturn IOFireWireAVCProtocolUserClient::clientDied( void )
  **/
 bool IOFireWireAVCProtocolUserClient::matchPropertyTable(OSDictionary * table)
 {
-    //
+    //IOLog( "IOFireWireAVCProtocolUserClient::matchPropertyTable (0x%08X)\n",(int) this);
+
+	//
     // If the service object wishes to compare some of its properties in its
     // property table against the supplied matching dictionary,
     // it should do so in this method and return truth on success.
@@ -336,7 +453,9 @@ bool IOFireWireAVCProtocolUserClient::matchPropertyTable(OSDictionary * table)
 
 IOExternalMethod* IOFireWireAVCProtocolUserClient::getTargetAndMethodForIndex(IOService **target, UInt32 index)
 {
-    if( index >= kIOFWAVCProtocolUserClientNumCommands )
+    //IOLog( "IOFireWireAVCProtocolUserClient::getTargetAndMethodForIndex (0x%08X)\n",(int) this);
+
+	if( index >= kIOFWAVCProtocolUserClientNumCommands )
         return NULL;
     else
     {
@@ -348,7 +467,9 @@ IOExternalMethod* IOFireWireAVCProtocolUserClient::getTargetAndMethodForIndex(IO
 IOExternalAsyncMethod* 
 IOFireWireAVCProtocolUserClient::getAsyncTargetAndMethodForIndex(IOService **target, UInt32 index)
 {
-    if( index >= kIOFWAVCProtocolUserClientNumAsyncCommands )
+    //IOLog( "IOFireWireAVCProtocolUserClient::getAsyncTargetAndMethodForIndex (0x%08X)\n",(int) this);
+
+	if( index >= kIOFWAVCProtocolUserClientNumAsyncCommands )
        return NULL;
    else
    {
@@ -361,25 +482,18 @@ IOFireWireAVCProtocolUserClient::getAsyncTargetAndMethodForIndex(IOService **tar
 IOReturn IOFireWireAVCProtocolUserClient::setAVCRequestCallback
 	( OSAsyncReference asyncRef, UInt32 subUnitType, UInt32 subUnitID)
 {
-    bcopy( asyncRef, fAVCRequestCallbackAsyncRef, sizeof(OSAsyncReference) );
+	//IOLog( "IOFireWireAVCProtocolUserClient::setAVCRequestCallback (0x%08X)\n",(int) this);
 
-    if(fAVCSpace) {
-        fAVCSpace->deactivate();
-        fAVCSpace->release();
-    }
-    
-    fAVCSpace = IOFireWireAVCRequestSpace::withSubUnit(fBus,
-                subUnitType, subUnitID, avcRequestHandler, this);
-    if(fAVCSpace)
-        fAVCSpace->activate();
-    
-    return kIOReturnSuccess;
+	// This function has been deprecated!	
+	return kIOReturnUnsupported;
 }
 
 IOReturn
 IOFireWireAVCProtocolUserClient::sendAVCResponse(UInt32 generation, UInt16 nodeID, const char *buffer, UInt32 size)
 {
-    IOMemoryDescriptor *desc = NULL;
+	//IOLog( "IOFireWireAVCProtocolUserClient::sendAVCResponse (0x%08X)\n",(int) this);
+
+	IOMemoryDescriptor *desc = NULL;
     IOFWWriteCommand *cmd = NULL;
     IOReturn status;
     do {
@@ -421,7 +535,9 @@ IOFireWireAVCProtocolUserClient::allocateInputPlug(OSAsyncReference asyncRef, vo
     IOReturn status;
     PlugInfo info;
     OSData *data;
-    
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::allocateInputPlug (0x%08X)\n",(int) this);
+	
     data = OSData::withCapacity(sizeof(info));
     if(!data)
         return kIOReturnNoMemory;
@@ -456,7 +572,9 @@ IOFireWireAVCProtocolUserClient::allocateOutputPlug(OSAsyncReference asyncRef, v
     IOReturn status;
     PlugInfo info;
     OSData *data;
-    
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::allocateOutputPlug (0x%08X)\n",(int) this);
+
     data = OSData::withCapacity(sizeof(info));
     if(!data)
         return kIOReturnNoMemory;
@@ -488,6 +606,9 @@ IOFireWireAVCProtocolUserClient::allocateOutputPlug(OSAsyncReference asyncRef, v
 IOReturn IOFireWireAVCProtocolUserClient::freeInputPlug(UInt32 plug)
 {
     OSIterator *plugIterator;
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::freeInputPlug (0x%08X)\n",(int) this);
+	
     plugIterator = OSCollectionIterator::withCollection(fInputPlugs);
     if( plugIterator) {
         OSData * data;
@@ -507,6 +628,9 @@ IOReturn IOFireWireAVCProtocolUserClient::freeInputPlug(UInt32 plug)
 IOReturn IOFireWireAVCProtocolUserClient::freeOutputPlug(UInt32 plug)
 {
     OSIterator *plugIterator;
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::freeOutputPlug (0x%08X)\n",(int) this);
+
     plugIterator = OSCollectionIterator::withCollection(fOutputPlugs);
     if( plugIterator) {
         OSData * data;
@@ -525,7 +649,9 @@ IOReturn IOFireWireAVCProtocolUserClient::freeOutputPlug(UInt32 plug)
 
 IOReturn IOFireWireAVCProtocolUserClient::readInputPlug(UInt32 plug, UInt32 *valPtr)
 {
-    if(plug > 30)
+	//IOLog( "IOFireWireAVCProtocolUserClient::readInputPlug (0x%08X)\n",(int) this);
+
+	if(plug > 30)
         return kIOReturnBadArgument;
         
     *valPtr = fPCRSpace->readInputPlug(plug);
@@ -534,6 +660,8 @@ IOReturn IOFireWireAVCProtocolUserClient::readInputPlug(UInt32 plug, UInt32 *val
 
 IOReturn IOFireWireAVCProtocolUserClient::updateInputPlug(UInt32 plug, UInt32 oldVal, UInt32 newVal)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::updateInputPlug (0x%08X)\n",(int) this);
+
     if(plug > 30)
         return kIOReturnBadArgument;
         
@@ -542,6 +670,8 @@ IOReturn IOFireWireAVCProtocolUserClient::updateInputPlug(UInt32 plug, UInt32 ol
 
 IOReturn IOFireWireAVCProtocolUserClient::readOutputPlug(UInt32 plug, UInt32 *valPtr)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::readOutputPlug (0x%08X)\n",(int) this);
+
     if(plug > 30)
         return kIOReturnBadArgument;
         
@@ -551,6 +681,8 @@ IOReturn IOFireWireAVCProtocolUserClient::readOutputPlug(UInt32 plug, UInt32 *va
 
 IOReturn IOFireWireAVCProtocolUserClient::updateOutputPlug(UInt32 plug, UInt32 oldVal, UInt32 newVal)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::updateOutputPlug (0x%08X)\n",(int) this);
+
     if(plug > 30)
         return kIOReturnBadArgument;
         
@@ -559,22 +691,148 @@ IOReturn IOFireWireAVCProtocolUserClient::updateOutputPlug(UInt32 plug, UInt32 o
 
 IOReturn IOFireWireAVCProtocolUserClient::readOutputMasterPlug(UInt32 *valPtr)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::readOutputMasterPlug (0x%08X)\n",(int) this);
+
     *valPtr = fPCRSpace->readOutputMasterPlug();
     return kIOReturnSuccess;
 }
 
 IOReturn IOFireWireAVCProtocolUserClient::updateOutputMasterPlug(UInt32 oldVal, UInt32 newVal)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::updateOutputMasterPlug (0x%08X)\n",(int) this);
+
     return fPCRSpace->updateOutputMasterPlug(oldVal, newVal);
 }
 
 IOReturn IOFireWireAVCProtocolUserClient::readInputMasterPlug(UInt32 *valPtr)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::readInputMasterPlug (0x%08X)\n",(int) this);
+
     *valPtr = fPCRSpace->readInputMasterPlug();
     return kIOReturnSuccess;
 }
 
 IOReturn IOFireWireAVCProtocolUserClient::updateInputMasterPlug(UInt32 oldVal, UInt32 newVal)
 {
+	//IOLog( "IOFireWireAVCProtocolUserClient::updateInputMasterPlug (0x%08X)\n",(int) this);
+
     return fPCRSpace->updateInputMasterPlug(oldVal, newVal);
 }
+
+IOReturn IOFireWireAVCProtocolUserClient::publishAVCUnitDirectory(void)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::publishAVCUnitDirectory (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->publishAVCUnitDirectory();
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::installAVCCommandHandler(OSAsyncReference asyncRef, UInt32 subUnitTypeAndID, UInt32 opCode, UInt32 callback, UInt32 refCon)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::installAVCCommandHandler (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->installAVCCommandHandler(this, avcTargetCommandHandler, asyncRef, subUnitTypeAndID, opCode, callback, refCon);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::addSubunit(OSAsyncReference asyncRef,
+													 UInt32 subunitType,
+													 UInt32 numSourcePlugs,
+													 UInt32 numDestPlugs,
+													 UInt32 callBack,
+													 UInt32 refCon,
+													 UInt32 *subUnitTypeAndID)
+{
+    IOReturn res = kIOReturnSuccess;
+	UInt32 subUnitID = 0x00;
+
+	//IOLog( "IOFireWireAVCProtocolUserClient::addSubunit (0x%08X)\n",(int) this);
+
+	res = fAVCTargetSpace->addSubunit(this,
+								   avcSubunitPlugHandler,
+								   asyncRef,
+								   subunitType,
+								   numSourcePlugs,
+								   numDestPlugs,
+								   callBack,
+								   refCon,
+								   &subUnitID);
+	if (res == kIOReturnSuccess)
+		*subUnitTypeAndID = subUnitID; 
+	
+    return res;
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::setSubunitPlugSignalFormat(UInt32 subunitTypeAndID,
+																	 IOFWAVCPlugTypes plugType,
+																	 UInt32 plugNum,
+																	 UInt32 signalFormat)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::setSubunitPlugSignalFormat (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->setSubunitPlugSignalFormat(this,subunitTypeAndID, plugType, plugNum, signalFormat);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::getSubunitPlugSignalFormat(UInt32 subunitTypeAndID,
+											IOFWAVCPlugTypes plugType,
+											UInt32 plugNum,
+											UInt32 *pSignalFormat)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::getSubunitPlugSignalFormat (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->getSubunitPlugSignalFormat(this,subunitTypeAndID, plugType, plugNum, pSignalFormat);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::connectTargetPlugs(AVCConnectTargetPlugsInParams *inParams,
+															 AVCConnectTargetPlugsOutParams *outParams)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::connectTargetPlugs (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->connectTargetPlugs(this,inParams,outParams);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::disconnectTargetPlugs(UInt32 sourceSubunitTypeAndID,
+									   IOFWAVCPlugTypes sourcePlugType,
+									   UInt32 sourcePlugNum,
+									   UInt32 destSubunitTypeAndID,
+									   IOFWAVCPlugTypes destPlugType,
+									   UInt32 destPlugNum)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::disconnectTargetPlugs (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->disconnectTargetPlugs(this,
+											   sourceSubunitTypeAndID,
+											   sourcePlugType,
+											   sourcePlugNum,
+											   destSubunitTypeAndID,
+											   destPlugType,
+											   destPlugNum);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::getTargetPlugConnection(AVCGetTargetPlugConnectionInParams *inParams,
+																  AVCGetTargetPlugConnectionOutParams *outParams)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::getTargetPlugConnection (0x%08X)\n",(int) this);
+
+	return fAVCTargetSpace->getTargetPlugConnection(this,inParams,outParams);
+}
+
+IOReturn IOFireWireAVCProtocolUserClient::AVCRequestNotHandled(UInt32 generation,
+									  UInt16 nodeID,
+									  IOFWSpeed speed,
+									  UInt32 handlerSearchIndex,
+									  const char *pCmdBuf,
+									  UInt32 cmdLen)
+{
+	//IOLog( "IOFireWireAVCProtocolUserClient::AVCRequestNotHandled (0x%08X)\n",(int) this);
+
+	fAVCTargetSpace->findAVCRequestHandler(this,
+							 generation,
+							 nodeID,
+							 speed,
+							 handlerSearchIndex,
+							 pCmdBuf,
+							 cmdLen);
+
+	return kIOReturnSuccess;
+}
+
+
